package net.wanji.utc.hisense.service.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.hisense.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.common.HiseseDirEnum;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.common.enums.CrossLisghtsLaneDirEnum;
import net.wanji.utc.hisense.common.enums.DataBrainControlModeEnum;
import net.wanji.utc.hisense.common.enums.DataBrainLampStatusEnum;
import net.wanji.utc.hisense.common.enums.DataBrainTurnEnum;
import net.wanji.utc.hisense.pojo.CrossPhaseDirTurnPojo;
import net.wanji.utc.hisense.pojo.convert.LightStatusPojo;
import net.wanji.utc.hisense.pojo.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.hisense.pojo.dto.PhaseCountDownDTO;
import net.wanji.utc.hisense.service.SignalStatusService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:28
 */
@Service
@Slf4j
public class SignalStatusServiceImpl implements SignalStatusService {
    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;

    @Value("${hisenseLightsStatus.shensiUrl}")
    private  String shensiUrl;
    @Value("${hisenseLightsStatus.baiduUrl}")
    private  String baiduUrl;
    @Value("${hisenseLightsStatus.yiGouUrl}")
    private  String yiGouUrl;

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) throws DocumentException {
        //从缓存中拿路口信息
        CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_3);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPO.getCode());
        List<SignalStatusLogPO> list = new ArrayList<>();

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.4. 联机状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            //Status：联机状态。0-离线，1-在线。
            String status = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Status");
            SignalStatusLogPO signalStatusLogPO = new SignalStatusLogPO();
            signalStatusLogPO.setCrossId(crossId);
            signalStatusLogPO.setSignalId(crossInfoPO.getCode());
            signalStatusLogPO.setStatus(Integer.valueOf(status));
            if ("1".equals(status)) {
                signalStatusLogPO.setFaultType(0);
            }else {
                signalStatusLogPO.setFaultType(5);
            }
            signalStatusLogPO.setControlType(null);
            signalStatusLogPO.setGmtCreate(null);
            list.add(signalStatusLogPO);

            return list;

        } else {
            //失败
            log.error("请求 5.4. 联机状态 失败");
        }

        return null;
    }

    /**
     * http 形式
     *
     * @param crossId
     * @return
     */
    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOS = new ArrayList<>();
        Map<String, LightsStatusVO> runningStateInfoCache = SignalDataCache.runningStateInfoCache;
        if (!runningStateInfoCache.isEmpty()) {
            LightsStatusVO lightsStatusVO = runningStateInfoCache.get(crossId);
            lightsStatusVOS.add(lightsStatusVO);
            return lightsStatusVOS;
        }
        return lightsStatusVOS;
    }

    /**
     * @param lightStatusPojos
     * @throws Exception
     * @description 通过http形式接收海信灯态
     */
    @Override
    public void receiveLightStatus(List<LightStatusPojo> lightStatusPojos) throws Exception {
        Date date = new Date();
        Long time = date.getTime();
        ObjectMapper jackson = JacksonUtils.getInstance();
        pushOtherCompany(lightStatusPojos, jackson, shensiUrl, baiduUrl, yiGouUrl, BaseEnum.VendorTypeEnum.HS.getNick());

        if (!CollectionUtils.isEmpty(lightStatusPojos)) {
            for (LightStatusPojo lightStatusPojo : lightStatusPojos) {
                String bodyStr = lightStatusPojo.getBody();
                List<LightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<LightStatusPojo.Body>>() {});
                if (!CollectionUtils.isEmpty(bodies)) {
                    for (LightStatusPojo.Body body : bodies) {
                        LightStatusPojo.ContentBody content = body.getContent();
                        LightsStatusVO lightsStatusVO = new LightsStatusVO();
                        String hiCrossId = content.getCrossId();
                        CrossInfoPO crossInfo = CrossInfoCache.getCrossInfoBySignalCode(hiCrossId);
                        Long signalMachineStamp = content.getSignalMachineStamp();
                        if (Objects.isNull(crossInfo)) {
                            continue;
                        }
                        StaticInfoServiceImpl.planMap.put(crossInfo.getId(), content.getPlanId());
                        String crossId = crossInfo.getId();
                        Long lastSignalMachineStamp = SignalDataCache.runningStatusStampMap.get(crossId);
                        if (Objects.isNull(lastSignalMachineStamp)) {
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        } else {
                            if (lastSignalMachineStamp >= signalMachineStamp) {
                                continue;
                            }
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        }

                        int offsetTime = (int) ((time - signalMachineStamp) / 1000);
                        long currentTimeMillis = System.currentTimeMillis();
                        log.info("灯态延迟时间：{}秒", offsetTime);
                        int cycleCountDown = content.getCycleCountDown() - Math.max(offsetTime, 0);
                        int hiControlMode = content.getControlMode();
                        int period = content.getPeriod();
                        int runTime = period - cycleCountDown;
                        int planId = (content.getPlanId() + 2) / 3;
                        lightsStatusVO.setCrossId(crossId);
                        lightsStatusVO.setCode(crossInfo.getCode());
                        lightsStatusVO.setSchemeId(String.valueOf(planId));
                        String runMode = String.valueOf(DataBrainControlModeEnum.getWjControlMode(hiControlMode));
                        lightsStatusVO.setRunMode(runMode);
                        lightsStatusVO.setCycleCountDown(cycleCountDown);
                        lightsStatusVO.setCycleLen(period);
                        lightsStatusVO.setSchemeStartTime(String.valueOf(signalMachineStamp));
                        lightsStatusVO.setTimeStamp(String.valueOf(signalMachineStamp + offsetTime));
                        lightsStatusVO.setPhaseMap(Collections.emptyMap());
                        lightsStatusVO.setControlMode(runMode);
                        lightsStatusVO.setCode(hiCrossId);
                        lightsStatusVO.setPhasePlanId(String.valueOf(planId));
                        lightsStatusVO.setManufacturerCode(BaseEnum.VendorTypeEnum.HS.getNick());
                        // 海信每5秒推送
                        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
                        LightsStatusVO lightsStatusVOCurrent = setLightsStatusVOPhaseNoAndCountDown(lightsStatusVO, crossId, runTime, planId, crossSchemePhaseCountDownList);
                        if (Objects.isNull(lightsStatusVOCurrent)) {
                            log.error("当前海信路口灯态信息异常：{}", lightsStatusVO);
                            continue;
                        }
                        SignalDataCache.runningStateInfoCacheNoLamp.put(crossId, lightsStatusVOCurrent);
                    }
                }
            }
        }
    }

    @Override
    public void receiveLightStatusEhualu(List<LightStatusPojo> ehualuLightStatusPojos) throws Exception {
        Date date = new Date();
        Long time = date.getTime();
        ObjectMapper jackson = JacksonUtils.getInstance();
        pushOtherCompany(ehualuLightStatusPojos, jackson, shensiUrl, baiduUrl, yiGouUrl, BaseEnum.VendorTypeEnum.EHUALU.getNick());

        if (!CollectionUtils.isEmpty(ehualuLightStatusPojos)) {
            for (LightStatusPojo LightStatusPojo : ehualuLightStatusPojos) {
                String bodyStr = LightStatusPojo.getBody();
                List<LightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<LightStatusPojo.Body>>() {});
                if (!CollectionUtils.isEmpty(bodies)) {
                    for (LightStatusPojo.Body body : bodies) {
                        LightStatusPojo.ContentBody content = body.getContent();
                        LightsStatusVO lightsStatusVO = new LightsStatusVO();
                        String hiCrossId = content.getCrossId();
                        CrossInfoPO crossInfo = CrossInfoCache.getCrossInfoBySignalCode(hiCrossId);
                        Long signalMachineStamp = content.getSignalMachineStamp();
                        if (Objects.isNull(crossInfo)) {
                            continue;
                        }
                        StaticInfoServiceImpl.planMap.put(crossInfo.getId(), content.getPlanId());
                        String crossId = crossInfo.getId();
                        Long lastSignalMachineStamp = SignalDataCache.runningStatusStampMap.get(crossId);
                        if (Objects.isNull(lastSignalMachineStamp)) {
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        } else {
                            if (lastSignalMachineStamp >= signalMachineStamp) {
                                continue;
                            }
                            SignalDataCache.runningStatusStampMap.put(crossId, signalMachineStamp);
                        }

                        int offsetTime = (int) ((time - signalMachineStamp) / 1000);
                        long currentTimeMillis = System.currentTimeMillis();
                        log.info("灯态延迟时间：{}秒", offsetTime);
                        int cycleCountDown = content.getCycleCountDown() - Math.max(offsetTime, 0);
                        int hiControlMode = content.getControlMode();
                        int period = content.getPeriod();
                        int runTime = period - cycleCountDown;
                        int planId = (content.getPlanId() + 2) / 3;
                        lightsStatusVO.setCrossId(crossId);
                        lightsStatusVO.setCode(crossInfo.getCode());
                        lightsStatusVO.setSchemeId(String.valueOf(planId));
                        String runMode = String.valueOf(DataBrainControlModeEnum.getWjControlMode(hiControlMode));
                        lightsStatusVO.setRunMode(runMode);
                        lightsStatusVO.setCycleCountDown(cycleCountDown);
                        lightsStatusVO.setCycleLen(period);
                        lightsStatusVO.setSchemeStartTime(String.valueOf(signalMachineStamp));
                        lightsStatusVO.setTimeStamp(String.valueOf(signalMachineStamp + offsetTime));
                        lightsStatusVO.setPhaseMap(Collections.emptyMap());
                        lightsStatusVO.setControlMode(runMode);
                        lightsStatusVO.setCode(hiCrossId);
                        lightsStatusVO.setPhasePlanId(String.valueOf(planId));
                        lightsStatusVO.setManufacturerCode(BaseEnum.VendorTypeEnum.EHUALU.getNick());
                        // 易华录每1秒推送
                            List<LightStatusPojo.ContentBody.LampRealInfos> lampRealInfos = content.getLampRealInfos();
                            String currentPhaseId = "";
                            Integer curCycleCountDown = 0;
                            if (!CollectionUtils.isEmpty(lampRealInfos)) {
                                Map<String, Object> dirLampGroupMap = new HashMap<>();
                                Map<String, Object> phaseMap = new HashMap<>();
                                Map<Integer, List<LightStatusPojo.ContentBody.LampRealInfos>> dirGroupMap = lampRealInfos.stream()
                                        .collect(Collectors.groupingBy(net.wanji.utc.hisense.pojo.convert.LightStatusPojo.ContentBody.LampRealInfos::getRoadDirection));
                                for (Map.Entry<Integer, List<LightStatusPojo.ContentBody.LampRealInfos>> entry : dirGroupMap.entrySet()) {
                                    Integer roadDir = entry.getKey();
                                    Integer dir = HiseseDirEnum.getWjLightsDir(roadDir);
                                    Map<Integer, String> turnColor = new HashMap<>();
                                    Map<Integer, Integer> turnCountDown = new HashMap<>();
                                    List<LightStatusPojo.ContentBody.LampRealInfos> value = entry.getValue();
                                    for (LightStatusPojo.ContentBody.LampRealInfos lampRealInfo : value) {
                                        String lampStatus = lampRealInfo.getLampStatus();
                                        String wjLampStatus = DataBrainLampStatusEnum.getWjLampStatus(lampStatus);
                                        Integer countDown = lampRealInfo.getCountDown();
                                        String phaseId = lampRealInfo.getPhaseId();
                                        Integer roadDirection = lampRealInfo.getRoadDirection();
                                        Integer roadFlowDirection = lampRealInfo.getRoadFlowDirection();
                                        Integer wjTurn = DataBrainTurnEnum.getWjTurn(roadFlowDirection);
                                        turnColor.put(wjTurn, wjLampStatus);
                                        if (!StringUtils.equalsIgnoreCase(DataBrainLampStatusEnum.RED.getWjLampStatus(), wjLampStatus)) {
                                            currentPhaseId = phaseId;
                                            curCycleCountDown = countDown;
                                        }
                                        turnCountDown.put(wjTurn, countDown);
                                    }
                                    phaseMap.put(String.valueOf(dir), turnCountDown);
                                    dirLampGroupMap.put(String.valueOf(dir), turnColor);
                                    lightsStatusVO.setPhaseMap(phaseMap);
                                    lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
                                }
                            }
                            lightsStatusVO.setPhasePlanId(currentPhaseId);
                            lightsStatusVO.setPhaseId(currentPhaseId);
                            lightsStatusVO.setCycleCountDown(curCycleCountDown);
                            lightsStatusVO.setCyclePhaseCountDown(curCycleCountDown);
                            SignalDataCache.runningStateInfoCache.put(crossId, lightsStatusVO);
                            log.error("当前路口灯态：{}", lightsStatusVO);
                    }
                }
            }
        }
    }

    private static void pushOtherCompany(Object hisenseLightStatusPojos, ObjectMapper jackson,
                                         String shensiUrl, String baiduUrl, String yiGouUrl, String type) {
        try {
            OkHttpClientUtil.jsonPost(shensiUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
        } catch (Exception e) {
            log.error("海信灯态推送神思失败：{}", e.getMessage());
        }
        try {
            OkHttpClientUtil.jsonPost(baiduUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
        } catch (Exception e) {
            log.error("海信灯态推送百度失败：{}", e.getMessage());
        }
        try {
            if (StringUtils.equalsIgnoreCase(type, BaseEnum.VendorTypeEnum.HS.getNick())) {
                List<LightStatusPojo> result = (List<LightStatusPojo>) hisenseLightStatusPojos;
                LightStatusPojo lightStatusPojo = result.get(0);
                String bodyStr = lightStatusPojo.getBody();
                List<LightStatusPojo.Body> bodies = jackson.readValue(bodyStr, new TypeReference<List<LightStatusPojo.Body>>() {});
                for (LightStatusPojo.Body body : bodies) {
                    LightStatusPojo.ContentBody content = body.getContent();
                    String crossId = content.getCrossId();
                    List<String> list = Arrays.asList("255136", "209723", "198207", "132163");
                    if (list.contains(crossId)) {
                        OkHttpClientUtil.jsonPost(yiGouUrl, jackson.writeValueAsString(hisenseLightStatusPojos));
                    }
                }
            }
        } catch (Exception e) {
            log.error("海信灯态推送易购失败：{}", e);
        }
    }

    /**
     * 通过周期倒计时判断当前相位号，相位倒计时时间
     *
     * @param lightsStatusVO
     * @param crossId
     * @param runTime
     * @param planId
     * @param crossSchemePhaseCountDownList
     */
    private static LightsStatusVO setLightsStatusVOPhaseNoAndCountDown(LightsStatusVO lightsStatusVO, String crossId, int runTime, int planId, List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList) {
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            LightsStatusVO lightsStatusVOTemp = new LightsStatusVO();
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                String crossIdCache = dto.getCrossId();
                String schemeNoCache = dto.getSchemeNo();
                if (StringUtils.equalsIgnoreCase(crossIdCache, crossId) && StringUtils.equalsIgnoreCase(schemeNoCache, String.valueOf(planId))) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    int lastPhaseCountTime = 0;
                    int phaseTimeCountDown = 0;
                    String phaseNo = null;
                    for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownDTOList) {
                        phaseNo = phaseCountDownDTO.getPhaseNo();
                        Integer changePhaseTime = phaseCountDownDTO.getChangePhaseTime();
                        if (lastPhaseCountTime <= runTime && runTime < changePhaseTime) {
                            phaseTimeCountDown = changePhaseTime - runTime;
                            break;
                        }
                        lastPhaseCountTime = changePhaseTime;
                    }
                    BeanUtils.copyProperties(lightsStatusVO, lightsStatusVOTemp);
                    lightsStatusVOTemp.setPhaseId(phaseNo);
                    lightsStatusVOTemp.setCyclePhaseCountDown(phaseTimeCountDown);
                    return lightsStatusVOTemp;
                }
            }
        }
        return null;
    }

    /**
     * @description 通过方向转向获取路口方案的相位号
     */
    private static Integer getCurrentPhagetseNo(String crossId, int planId, Map<Integer, List<Integer>> hisDirTurnMap) {
        Integer phaseNo = null;
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnList = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnList)) {
            List<CrossPhaseDirTurnPojo> schemeNoList = crossPhaseDirTurnList.stream().filter(po -> Objects.equals(po.getSchemeNo(), planId)).collect(Collectors.toList());
            for (CrossPhaseDirTurnPojo crossPhaseDirTurnPojo : schemeNoList) {
//                Map<Integer, List<Integer>> dirTurnCache = crossPhaseDirTurnPojo.getDirTurnMap();
                Map<Integer, List<CrossLightsPO>> dirTurnMap = crossPhaseDirTurnPojo.getDirTurnMap();
                for (Map.Entry<Integer, List<CrossLightsPO>> dirTurnCacheEntry : dirTurnMap.entrySet()) {
                    Integer dirCache = dirTurnCacheEntry.getKey();
                    for (Map.Entry<Integer, List<Integer>> hisEntry : hisDirTurnMap.entrySet()) {
                        Integer hisDir = hisEntry.getKey();
                        if (Objects.equals(dirCache, CrossLisghtsLaneDirEnum.getLightsDirByLaneDir(hisDir))) {
                            List<CrossLightsPO> turnListCache = dirTurnCacheEntry.getValue();
                            List<Integer> turnList = turnListCache.stream().map(CrossLightsPO::getTurn).sorted().collect(Collectors.toList());
                            if (hisEntry.getValue().stream().allMatch(turnList::contains)) {
                                phaseNo = crossPhaseDirTurnPojo.getPhaseNo();
                            }
                        }
                    }
                }
            }
        }
        return phaseNo;
    }

}
