package net.wanji.utc.hisense.task;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossLightsPO;
import net.wanji.databus.dao.entity.CrossPhasePO;
import net.wanji.databus.dao.entity.CrossSchemePO;
import net.wanji.databus.dao.mapper.CrossLightsMapper;
import net.wanji.databus.dao.mapper.CrossPhaseMapper;
import net.wanji.databus.dao.mapper.CrossSchemeMapper;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.hisense.cache.CrossRunSchemeCache;
import net.wanji.utc.hisense.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.hisense.cache.SignalDataCache;
import net.wanji.utc.hisense.pojo.CrossPhaseDirTurnPojo;
import net.wanji.utc.hisense.pojo.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.hisense.pojo.dto.PhaseCountDownDTO;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/09/20 14:25
 */
@Component
@Slf4j
public class HisensePhaseCountDownTask {

    @Resource
    private CrossPhaseMapper crossPhaseMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;
    @Resource
    private CrossLightsMapper crossLightsMapper;
    @Resource(name = "commonThreadPoolExecutor")
    ThreadPoolTaskExecutor commonThreadPoolExecutor;

    @Scheduled(fixedRate = 1000, initialDelay = 1000)
    public void phaseTimeCountDown() throws Exception {
        commonThreadPoolExecutor.execute(() -> {
            try {
                Map<String, LightsStatusVO> runningStateInfoCacheNoLamp = SignalDataCache.runningStateInfoCacheNoLamp;
                if (!runningStateInfoCacheNoLamp.isEmpty()) {
                    for (Map.Entry<String, LightsStatusVO> entry : runningStateInfoCacheNoLamp.entrySet()) {
                        String crossId = entry.getKey();
                        LightsStatusVO lightsStatusVO = entry.getValue();
                        long timeStamp = Long.parseLong(lightsStatusVO.getTimeStamp());
                        String schemeStartTime = lightsStatusVO.getSchemeStartTime();
                        String phaseId = lightsStatusVO.getPhaseId();
                        String schemeId = lightsStatusVO.getSchemeId();
                        CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(schemeId));

                        // 通过时段表，切换方案 凌晨00：00 如果切换下一个日期方案，可能RefreshCacheTask还没执行
                        String nextSectionSchemeNo = CrossRunSchemeCache.currentRunSchemeNoCache.get(crossId);
                        Date currentDate = new Date();
                        String currentTime = String.valueOf(currentDate.getTime());
                        Date date2HoursAgo = DateUtils.addHours(currentDate, -1);
                        if (StringUtils.equalsIgnoreCase(schemeStartTime, currentTime) || Long.parseLong(schemeStartTime) < date2HoursAgo.getTime()) {
                            if (StringUtils.isNotBlank(nextSectionSchemeNo)) {
                                crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(nextSectionSchemeNo));
                                executeNextSectionScheme(crossId, lightsStatusVO, crossSchemePO);
                                lightsStatusVO.setCycleLen(crossSchemePO.getCycle());
                            }
                        } else {
                            executeNextPeriod(crossId, lightsStatusVO, phaseId, crossSchemePO);
                        }
                        // 当前方案下一周期
                        setDirLampGroupMap(lightsStatusVO);
                        if (Objects.nonNull(lightsStatusVO.getDirLampGroupMap())) {
                            if (Objects.equals(crossId, "13NG10B5RH0")) {
                                List<String> dirList = Arrays.asList("1", "3", "7");
                                Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
                                for (String dir : dirList) {
                                    if (!dirLampGroupMap.containsKey(dir)) {
                                        Map<Integer, String> value = new HashMap<>();
                                        setAllTurnColorRed(value, null, null);
                                        dirLampGroupMap.put(dir, value);
                                    }
                                }
                                lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
                            }
                            // 当前时间戳自动加1s
                            lightsStatusVO.setTimeStamp(String.valueOf(timeStamp + 1000));
                            // 红灯倒计时
                            getRedCountDown(crossId, lightsStatusVO, schemeId);
                            SignalDataCache.runningStateInfoCache.put(crossId, lightsStatusVO);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("定时任务计算路口相位灯态倒计时异常", e);
                throw new RuntimeException(e);
            }
        });

    }

    private static void getRedCountDown(String crossId, LightsStatusVO lightsStatusVO, String schemeId) {
        Map<String, Object> phaseMap = new HashMap<>();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        List<PhaseCountDownDTO> phaseCountDownList = getPhaseCountDownList(crossId, schemeId);
        if (!CollectionUtils.isEmpty(phaseCountDownList)) {
            for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownList) {
                Integer changePhaseTime = phaseCountDownDTO.getChangePhaseTime();
                Integer phaseTime = phaseCountDownDTO.getPhaseTime();
                Integer cycleLen = lightsStatusVO.getCycleLen();
                Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
                String currenPhaseId = lightsStatusVO.getPhaseId();
                int runtime = cycleLen - cycleCountDown;
                int redCountDown = 0;
                if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
                    for (CrossPhaseDirTurnPojo dirTurnPojo : crossPhaseDirTurnCache) {
                        // 当前方案
                        if (StringUtils.equalsIgnoreCase(schemeId, String.valueOf(dirTurnPojo.getSchemeNo()))) {
                            // 当前相位
                            if (!StringUtils.equalsIgnoreCase(currenPhaseId, String.valueOf(dirTurnPojo.getPhaseNo()))) {
                                Map<Integer, List<CrossLightsPO>> dirTurnMap = dirTurnPojo.getDirTurnMap();
                                if (ObjectUtil.isNotEmpty(dirTurnMap)) {
                                    for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
                                        Integer dir = dirEntry.getKey();
                                        Map<Integer, Integer> turnCountDown = new HashMap<>();
                                        List<CrossLightsPO> lightsPOS = dirEntry.getValue();
                                        if (!CollectionUtils.isEmpty(lightsPOS)) {
                                            // 忽略行人
                                            List<CrossLightsPO> person = lightsPOS.stream().filter(po -> po.getTurn() == 20).collect(Collectors.toList());
                                            if (person.size() == lightsPOS.size()) {
                                                continue;
                                            }
                                            Integer type = lightsPOS.get(0).getType();
                                            redCountDown = changePhaseTime - runtime - phaseTime;
                                            if (redCountDown <= 0) {
                                                redCountDown = cycleLen - runtime;
                                            }
                                            //if (redCountDown <= 0) {
                                            //    redCountDown = 1;
                                            //}
                                            if (type == 2) { // 圆饼灯
                                                List<Integer> dirList = Arrays.asList(1, 2, 3);
                                                for (Integer commonTurn : dirList) {
                                                    turnCountDown.put(commonTurn, redCountDown);
                                                }
                                            } else {
                                                Map<Integer, List<CrossLightsPO>> turnMap = lightsPOS.stream().collect(Collectors.groupingBy(CrossLightsPO::getTurn));
                                                for (Map.Entry<Integer, List<CrossLightsPO>> turnEntry : turnMap.entrySet()) {
                                                    Integer turn = turnEntry.getKey();
                                                    List<CrossLightsPO> value = turnEntry.getValue();
                                                    for (CrossLightsPO lightsPO : value) {
                                                        if (lightsPO.getTurn() != 20) {
                                                            turnCountDown.put(turn, redCountDown);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        setPhaseMap(phaseMap, dir, turnCountDown);
                                    }
                                }

                            } else {
                                // 当前方案当前相位绿灯
                                Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
                                Map<Integer, List<CrossLightsPO>> dirTurnMap = dirTurnPojo.getDirTurnMap();
                                if (ObjectUtil.isNotEmpty(dirTurnMap)) {
                                    for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
                                        Integer dir = dirEntry.getKey();
                                        Map<Integer, Integer> turnCountDown = new HashMap<>();
                                        List<CrossLightsPO> lightsPOS = dirEntry.getValue();
                                        if (!CollectionUtils.isEmpty(lightsPOS)) {
                                            // 忽略行人
                                            List<CrossLightsPO> person = lightsPOS.stream().filter(po -> po.getTurn() == 20).collect(Collectors.toList());
                                            if (person.size() == lightsPOS.size()) {
                                                continue;
                                            }
                                            Integer type = lightsPOS.get(0).getType();
                                             Integer greenCountDown = cyclePhaseCountDown;
                                            if (type == 2) { // 圆饼灯
                                                List<Integer> dirList = Arrays.asList(1, 2, 3);
                                                for (Integer commonTurn : dirList) {
                                                    turnCountDown.put(commonTurn, greenCountDown);
                                                }
                                            } else {
                                                Map<Integer, List<CrossLightsPO>> turnMap = lightsPOS.stream().collect(Collectors.groupingBy(CrossLightsPO::getTurn));
                                                for (Map.Entry<Integer, List<CrossLightsPO>> turnEntry : turnMap.entrySet()) {
                                                    Integer turn = turnEntry.getKey();
                                                    List<CrossLightsPO> value = turnEntry.getValue();
                                                    for (CrossLightsPO lightsPO : value) {
                                                        if (lightsPO.getTurn() != 20) {
                                                            turnCountDown.put(turn, greenCountDown);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        setPhaseMap(phaseMap, dir, turnCountDown);
                                    }
                                }

                            }
                        }
                    }
                }
            }
        }
        lightsStatusVO.setPhaseMap(phaseMap);
    }

    /**
     * 统一方向转向倒计时
     *
     * @param phaseMap
     * @param dir
     * @param turnCountDown
     */
    private static void setPhaseMap(Map<String, Object> phaseMap, Integer dir, Map<Integer, Integer> turnCountDown) {
        String dirStr = String.valueOf(dir);
        if (!phaseMap.isEmpty() && Objects.nonNull(phaseMap.get(dirStr))) {
            Map<Integer, Integer> turnCountDownCache = (Map<Integer, Integer>) phaseMap.get(dirStr);
            turnCountDown.putAll(turnCountDownCache);
            phaseMap.put(String.valueOf(dir), turnCountDown);
        } else {
            phaseMap.put(String.valueOf(dir), turnCountDown);
        }
    }

    /**
     * @param crossId
     * @param lightsStatusVO
     * @param crossSchemePO
     * @description 通过日计划缓存，切换下一个时段方案
     */
    private void executeNextSectionScheme(String crossId, LightsStatusVO lightsStatusVO, CrossSchemePO crossSchemePO) throws Exception {
        if (Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> nextSchemePhaseList = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer phaseTime = 0;
            String phaseId = null;

            if (!CollectionUtils.isEmpty(nextSchemePhaseList)) {
                for (CrossPhasePO crossPhasePO : nextSchemePhaseList) {
                    if (Objects.equals(1, crossPhasePO.getSort())) {
                        phaseTime = crossPhasePO.getPhaseTime();
                        phaseId = crossPhasePO.getPhaseNo();
                    }
                }
            }
            lightsStatusVO.setPhaseId(phaseId);
            lightsStatusVO.setCyclePhaseCountDown(phaseTime);
        }
    }

    /**
     * @param crossId
     * @param lightsStatusVO
     * @param phaseId
     * @param crossSchemePO
     * @description 切换下一方案周期
     */
    private void executeNextPeriod(String crossId, LightsStatusVO lightsStatusVO, String phaseId, CrossSchemePO crossSchemePO) throws Exception {
        if (lightsStatusVO.getCyclePhaseCountDown() <= 1 && Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer nextPhaseTime = 0;
            String nextPhaseId = "0";
            if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                for (int i = 0; i < crossPhasePOS.size(); i++) {
                    if (StringUtils.equalsIgnoreCase(phaseId, crossPhasePOS.get(i).getPhaseNo())) {
                        if (i == crossPhasePOS.size() - 1) {
                            nextPhaseTime = crossPhasePOS.get(0).getPhaseTime();
                            nextPhaseId = crossPhasePOS.get(0).getPhaseNo();
                        } else {
                            nextPhaseTime = crossPhasePOS.get(i + 1).getPhaseTime();
                            nextPhaseId = crossPhasePOS.get(i + 1).getPhaseNo();
                        }
                    }
                }
            }
            lightsStatusVO.setPhaseId(nextPhaseId);
            lightsStatusVO.setCyclePhaseCountDown(nextPhaseTime);
        } else {
            lightsStatusVO.setCyclePhaseCountDown(lightsStatusVO.getCyclePhaseCountDown() - 1);
            Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
            lightsStatusVO.setCycleCountDown(--cycleCountDown);
        }
    }

    /**
     * @param lightsStatusVO
     * @description 通过当前周期倒计时方案相位灯态
     */
    public void setDirLampGroupMap(LightsStatusVO lightsStatusVO) throws Exception {
        String crossId = lightsStatusVO.getCrossId();
        String currentSchemeNo = lightsStatusVO.getSchemeId();
        String currentPhaseNo = lightsStatusVO.getPhaseId();
        Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
            Map<String, Object> dirLampGroupMap = new HashMap<>();
            setPhaseLampColor(crossId, currentSchemeNo, currentPhaseNo, cyclePhaseCountDown, crossPhaseDirTurnCache, dirLampGroupMap);
            lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
        }
    }

    private void setPhaseLampColor(String crossId, String currentSchemeNo, String currentPhaseNo, Integer cyclePhaseCountDown, List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache, Map<String, Object> dirLampGroupMap) throws Exception {
        for (CrossPhaseDirTurnPojo phaseDirTurnPojo : crossPhaseDirTurnCache) {
            String schemeNo = String.valueOf(phaseDirTurnPojo.getSchemeNo());
            String phaseNo = String.valueOf(phaseDirTurnPojo.getPhaseNo());
            if (StringUtils.equalsIgnoreCase(schemeNo, currentSchemeNo)) {
                // 判断圆饼灯代表123，还是23
                List<CrossLightsPO> crossLightsPOS = crossLightsMapper.selectByCrossId(crossId);
                if (!StringUtils.equalsIgnoreCase(currentPhaseNo, String.valueOf(phaseNo))) {
                    // 相位红灯
                    Map<Integer, List<CrossLightsPO>> dirTurnMap = phaseDirTurnPojo.getDirTurnMap();
                    if (ObjectUtil.isNotEmpty(dirTurnMap)) {
                        for (Map.Entry<Integer, List<CrossLightsPO>> entry : dirTurnMap.entrySet()) {
                            String dir = String.valueOf(entry.getKey());
                            Map<Integer, String> turnColor = null;
                            if (!dirLampGroupMap.containsKey(dir)) {
                                turnColor = new HashMap<>();
                            } else {
                                turnColor = (Map<Integer, String>) dirLampGroupMap.get(dir);
                            }
                            List<CrossLightsPO> value = entry.getValue();
                            int type = 1;
                            for (CrossLightsPO crossLightsPO : value) { //todo 需要处理，路口方向级别，灯组圆饼灯要区分，有没有左转，crossd dir级别，有几个转向
                                Integer turn = crossLightsPO.getTurn();
                                if (crossLightsPO.getType() != 3) { // 设置灯色时不处理行人
                                    type = crossLightsPO.getType();
                                }
                                // 搭接相位跳过绿灯变红灯
                                if (turnColor.containsKey(turn) && turnColor.get(turn).equals("green")) {
                                    continue;
                                }
                                turnColor.put(turn, "red");
                            }
                            List<CrossLightsPO> dirAllLights = crossLightsPOS.stream()
                                    .filter(po -> Objects.equals(po.getDir(), entry.getKey()) && !Objects.equals(po.getType(), 3))
                                    .collect(Collectors.toList());
                            if (type == 2) {
                                if (dirAllLights.size() == 1) {
                                    setAllTurnColor(turnColor, "red"); //红灯时，圆饼灯操作
                                } else {
                                    turnColor.put(2, "red");
                                }
                            }
                            turnColor.put(3, "green");
                            dirLampGroupMap.put(dir, turnColor);
                        }
                    }
                } else {
                    // 相位绿灯
                    PhaseCountDownDTO phaseCountDownDTO = getPhaseCountDownDTO(crossId, schemeNo, phaseNo);
                    if (Objects.isNull(phaseCountDownDTO)) {
                        continue;
                    }
                    Integer yellowTime = phaseCountDownDTO.getYellowTime();
                    Integer redTime = phaseCountDownDTO.getRedTime();
                    Map<Integer, List<CrossLightsPO>> dirTurnMap = phaseDirTurnPojo.getDirTurnMap();
                    //if (ObjectUtil.isNotEmpty(dirTurnMap)) {
                    //    setDirTurnColor(cyclePhaseCountDown, dirLampGroupMap, yellowTime, redTime, dirTurnMap, crossLightsPOS);
                    //} else {
                    //    log.error("路口:{},dirTurnMap为空,{}", crossId, dirTurnMap);
                    //}
                    for (Map.Entry<Integer, List<CrossLightsPO>> entry : dirTurnMap.entrySet()) {
                        String dir = String.valueOf(entry.getKey());
                        Map<Integer, String> turnColor = null;
                        if (!dirLampGroupMap.containsKey(dir)) {
                            turnColor = new HashMap<>();
                        } else {
                            turnColor = (Map<Integer, String>) dirLampGroupMap.get(dir);
                        }
                        List<CrossLightsPO> value = entry.getValue();
                        int type = 1;
                        for (CrossLightsPO crossLightsPO : value) { //todo 需要处理，路口方向级别，灯组圆饼灯要区分，有没有左转，crossd dir级别，有几个转向
                            Integer turn = crossLightsPO.getTurn();
                            if (crossLightsPO.getType() != 3) { // 设置灯色时不处理行人
                                type = crossLightsPO.getType();
                            }
                            // 搭接相位跳过绿灯变红灯
                            if (turnColor.containsKey(turn) && turnColor.get(turn).equals("green")) {
                                continue;
                            }
                            turnColor.put(turn, "green");
                        }
                        List<CrossLightsPO> dirAllLights = crossLightsPOS.stream()
                                .filter(po -> Objects.equals(po.getDir(), entry.getKey()) && !Objects.equals(po.getType(), 3))
                                .collect(Collectors.toList());
                        if (type == 2) {
                            if (dirAllLights.size() == 1) {
                                setAllTurnColor(turnColor, "green"); //红灯时，圆饼灯操作
                            } else {
                                turnColor.put(2, "green");
                            }
                        }
                        turnColor.put(3, "green");
                        dirLampGroupMap.put(dir, turnColor);
                    }
                }
            }
        }
    }

    private static void setDirTurnColor(Integer cyclePhaseCountDown, Map<String, Object> dirLampGroupMap,
                                        Integer yellowTime, Integer redTime, Map<Integer,
            List<CrossLightsPO>> dirTurnMap, List<CrossLightsPO> crossLightsPOS) throws Exception {
        for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
            String dir = String.valueOf(dirEntry.getKey());
            List<CrossLightsPO> lightsList = dirEntry.getValue();
            if (CollectionUtils.isEmpty(lightsList)) {
                continue;
            }
            Map<Integer, String> turnColor = null;
            if (!dirLampGroupMap.containsKey(dir)) {
                turnColor = new HashMap<>();
            } else {
                turnColor = (Map<Integer, String>) dirLampGroupMap.get(dir);
            }
            // 灯组类型：1箭头；2圆饼；3行人；
            Integer type = 2;
            String color = null;
            for (CrossLightsPO crossLightsPO : lightsList) {
                type = crossLightsPO.getType();
                Integer turn = crossLightsPO.getTurn();
                color = setTurnColor(cyclePhaseCountDown, yellowTime, redTime, turnColor, color, turn);
                // 搭接相位跳过绿灯变红灯
                if (turnColor.containsKey(turn) && turnColor.get(turn).equals("green")) {
                    continue;
                }
                turnColor.put(turn, color);
            }
            List<CrossLightsPO> dirAllLights = crossLightsPOS.stream()
                    .filter(po -> Objects.equals(po.getDir(), dirEntry.getKey()) && !Objects.equals(po.getType(), 3))
                    .collect(Collectors.toList());
            if (type == 2) { // 方向是圆饼灯，并且只有一个灯组，不包含行人
                if (dirAllLights.size() == 1) {
                    setAllTurnColor(turnColor, color);
                } else {
                    turnColor.put(2, "green");
                    turnColor.put(3, "green"); // 右转常绿
                }
            }
            dirLampGroupMap.put(dir, turnColor);
        }
    }

    private static String setTurnColor(Integer cyclePhaseCountDown, Integer yellowTime, Integer redTime,
                                       Map<Integer, String> turnColor, String color, Integer turn) throws Exception {
        if (cyclePhaseCountDown > yellowTime + redTime) {
            color = "green";
            if (turn == 20) {
                turnColor.put(turn, "red");
            }
        }
        if (cyclePhaseCountDown <= yellowTime + redTime && cyclePhaseCountDown > redTime) {
            color = "yellow";
            if (turn == 20) {
                turnColor.put(turn, "red");
            }
        }
        if (cyclePhaseCountDown <= redTime) {
            color = "red";
            if (turn == 20) {
                turnColor.put(turn, "green");
            }
        }
        return color;
    }

    /**
     * 两秒倒计时，全场红
     *
     * @param turnColor
     * @param type
     * @param color
     */
    private static void setAllTurnColorRed(Map<Integer, String> turnColor, Integer type, String color) throws Exception {
        List<Integer> allTurn = Arrays.asList(1, 2, 3);
        for (Integer turnStr : allTurn) {
            if (!turnColor.containsKey(turnStr)) {
                turnColor.put(turnStr, "red");
            }
        }
        for (Map.Entry<Integer, String> entry : turnColor.entrySet()) {
            entry.setValue("red");
        }
    }

    /**
     * 圆饼灯设置所有方向灯色
     *
     * @param turnColor
     * @param color
     */
    private static void setAllTurnColor(Map<Integer, String> turnColor, String color) throws Exception {
        List<Integer> allTurn = Arrays.asList(1, 2);
        for (Integer turn : allTurn) {
            if (!turnColor.containsKey(turn)) {
                turnColor.put(turn, color);
            }
        }
        turnColor.put(3, "green"); // 右转常绿
    }

    /**
     * @param crossId
     * @param schemeNo
     * @param phaseNo
     * @return
     * @description 获取路口方案相位倒计时
     */
    private PhaseCountDownDTO getPhaseCountDownDTO(String crossId, String schemeNo, String phaseNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                if (Objects.equals(crossId, dto.getCrossId()) && Objects.equals(schemeNo, dto.getSchemeNo())) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    if (!CollectionUtils.isEmpty(phaseCountDownDTOList)) {
                        for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownDTOList) {
                            if (Objects.equals(phaseNo, phaseCountDownDTO.getPhaseNo())) {
                                return phaseCountDownDTO;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    private static List<PhaseCountDownDTO> getPhaseCountDownList(String crossId, String schemeNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                if (Objects.equals(crossId, dto.getCrossId()) && Objects.equals(schemeNo, dto.getSchemeNo())) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    if (!CollectionUtils.isEmpty(phaseCountDownDTOList)) {
                        return phaseCountDownDTOList;
                    }
                }
            }
        }
        return Collections.emptyList();
    }
}
