package net.wanji.utc.hisense.util;


import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.StringUtils;
import okhttp3.*;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName OkHttpClient
 * @Description OkHttpClient
 * @Author zhouleilei
 * @Date 2024/11/3 18:11
 */
@Slf4j
public class OkHttpClientUtil {

    public static String xmlPost(String url, String xmlContent) {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(xmlContent)) {
            return null;
        }
        // 创建 OkHttpClient 实例
        OkHttpClient client = new OkHttpClient();
        // 创建请求体
        RequestBody body = RequestBody.create(xmlContent, MediaType.get("application/xml; charset=utf-8"));
        // 创建请求
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        // 执行请求
        try (Response response = client.newCall(request).execute()) {
            // 获取响应体
            if (response.code() == 200 && response.body() != null) {
                String responseString = response.body().string();
                return responseString;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }


    public static String jsonPost(String url, String json) throws Exception {
        if (StringUtils.isBlank(url) || StringUtils.isBlank(json)) {
            return null;
        }
        // 创建 OkHttpClient 实例
        OkHttpClient client = new OkHttpClient().newBuilder()
                .connectTimeout(30, TimeUnit.SECONDS)
                .readTimeout(30, TimeUnit.SECONDS)
                .build();
        // 创建请求体
        RequestBody body = RequestBody.create(json, MediaType.get("application/json; charset=utf-8"));
        // 创建请求
        Request request = new Request.Builder()
                .url(url)
                .post(body)
                .build();
        // 执行请求
        try (Response response = client.newCall(request).execute()) {
            // 获取响应体
            if (response.code() == 200 && response.body() != null) {
                String responseString = response.body().string();
                log.info("远程服务调用成功，url：{}", url);
                return responseString;
            }
        } catch (Exception e) {
            log.error("OkHttpClientUtil远程服务url:{}, 调用异常：{}", url, e.getMessage());
            throw new Exception(e);
        }
        return null;
    }

    public static void main(String[] args) {

        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1730623180670</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>882530</Spot>\n" +
                "        <CoordPatternStatus>58</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>200</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>41</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";*/
        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription System=\"TCIP\" Version=\"1.0\"><subSystem>Hisense</subSystem><isRequest>1</isRequest><seq>20241103190736692398</seq><needResponse>1</needResponse><result>0</result><flag>0</flag><messageContent><Spot>882530</Spot></messageContent><messageContent>12</messageContent></systemScription>";
        String s = xmlPost("http://172.24.71.94:10013", post);
        System.out.println(s);
    }
}
